<?php

declare(strict_types=1);

namespace Drupal\Tests\jsonrpc\Functional;

use Drupal\Core\Url;
use Drupal\Tests\jsonrpc_discovery\Functional\JsonRpcDiscoveryFunctionalTestBase;
use Drupal\user\UserInterface;

/**
 * Tests the jsonrpc/methods endpoint.
 *
 * @group jsonrpc
 */
class JsonRpcDiscoveryHttpTest extends JsonRpcDiscoveryFunctionalTestBase {
  const PLUGINS_METHOD_NAME = 'List defined plugins';

  /**
   * Executes a request to jsonrpc/methods.
   *
   * @return string
   *   The absolute url.
   */
  protected function getMethodsUrl(): string {
    return Url::fromRoute('jsonrpc.method_collection')
      ->setAbsolute()->toString();
  }

  /**
   * Provides a basic auth header.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user account.
   *
   * @return string
   *   The basic auth header value formatted for Guzzle.
   */
  protected function getAuthForUser(UserInterface $user): string {
    $name = $user->getAccountName();
    $pass = $user->getPassword();
    return 'Basic ' . base64_encode($name . ':' . $pass);
  }

  /**
   * Tests getting the methods as an anonymous user.
   */
  public function testMethodsAnon(): void {
    // Anon does not have access to JSON-RPC services.
    $method_url = $this->getMethodsUrl();
    $anon_response = \Drupal::httpClient()->get($method_url, [
      'body' => NULL,
      'headers' => [],
    ]);
    $this->assertEquals(200, $anon_response->getStatusCode());
    // Anon does not have access to the plugins method.
    $this->assertStringNotContainsString(self::PLUGINS_METHOD_NAME, $anon_response->getBody()
      ->getContents());
  }

  /**
   * Tests getting the methods as an auth user.
   */
  public function testMethodsAuth(): void {
    $method_url = $this->getMethodsUrl();
    $auth_response = \Drupal::httpClient()->get($method_url, [
      'body' => NULL,
      'headers' => [
        'Authorization' => $this->getAuthForUser($this->user),
      ],
    ]);
    $this->assertEquals(200, $auth_response->getStatusCode());
    // Auth does not have access to the plugins method.
    $this->assertStringNotContainsString(self::PLUGINS_METHOD_NAME, $auth_response->getBody()
      ->getContents());
  }

}
